@@ -15,8 +15,7 @@
 
 (defun sepia-tree-button-cb (widget &rest blah)
   (let* ((pw (widget-get widget :parent))
-	 (location (mapcar (lambda (x) (widget-get pw x))
-			   '(:sepia-file :sepia-line :sepia-obj :sepia-mod))))
+	 (location (xref-location (widget-get widget :sepia-name))))
     (cond
       (current-prefix-arg
        (find-file-other-window (car location))
@@ -41,34 +40,26 @@
 (defun sepia-tree-node-cb (widget &rest blah)
   (let ((func (widget-get widget :sepia-func)))
     (or (widget-get widget :args)
-	(let ((children
-	       (sort
-		(sepia-uniquify (funcall func widget))
-		#'sepia-tree-def-order
-		)))
+	(let ((children (funcall func widget)))
 	  (if children
 	      (mapcar
-	       (lambda (x) (apply #'sepia-tree-node func x))
+	       (lambda (x) (sepia-tree-node func x))
 	       children)
 	      (widget-put widget :has-children nil))))))
 
-(defun sepia-tree-node (func file line obj mod)
+(defun sepia-tree-node (func name)
   "Make a tree node for the object specified by FILE, LINE, OBJ,
 and MOD.  The new node will have a property :sepia-X
 corresponding to each of these values.  FUNC is a function that
 will, given a widget, generate its children."
   `(tree-widget
     :node (push-button
-	   :tag ,(format "%s::%s" mod obj)
+	   :tag ,name
 	   :format "%[%t%]\n"
-	   :help-echo ,(format "%s::%s -- %s:%s" mod obj file line)
 	   :notify sepia-tree-button-cb)
     :dynargs sepia-tree-node-cb
     :has-children t
-    :sepia-obj ,obj
-    :sepia-mod ,mod
-    :sepia-file ,file
-    :sepia-line ,line
+    :sepia-name ,name
     :sepia-func ,func))
 
 (defun sepia-tree-tidy-buffer (name)
@@ -91,11 +82,10 @@ will, given a widget, generate its children."
 	(with-current-buffer bufname
 	  (dolist (x defs)
 	    (apply #'widget-create
-		   (apply #'sepia-tree-node
-			  (lambda (widget)
-			    (funcall func (widget-get widget :sepia-obj)
-				     (widget-get widget :sepia-mod)))
-			  x)))
+		   (sepia-tree-node
+                    (lambda (widget)
+                      (funcall func (widget-get widget :sepia-name)))
+                    x)))
 	  (use-local-map (copy-keymap widget-keymap))
 ;;	  (local-set-key "\M-." sepia-keymap)
 	  (sepia-install-keys)
@@ -105,56 +95,44 @@ will, given a widget, generate its children."
 	  (message "Type C-h m for usage information")))
       (message "No items for %s" bufname)))
 
-(defun sepia-callee-tree (obj mod)
+;;;###autoload
+(defun sepia-callee-tree (name)
   "Create a tree view of a function's callees.
 
 Pressing RET on a function's name displays its definition.  With
 prefix argument, RET instead visits in another window."
-  (interactive (list (sepia-interactive-arg 'function)
-		     (sepia-interactive-module)))
-  (let* ((defs (xref-defs obj mod))
-	 (mod (if (= (length defs) 1) (or (fourth (car defs)) mod) "*")))
-    (sepia-build-tree-buffer #'xref-callees defs
-			     (format "*%s::%s callees*" mod obj))))
+  (interactive (let ((func (sepia-interactive-arg 'function))
+                     (mod (sepia-interactive-module)))
+                 (list (if mod (format "%s::%s" mod func)
+                           func))))
+  (let* ((defs (xref-apropos name)))
+    (sepia-build-tree-buffer
+     #'xref-callees
+     defs
+     (format "*%s callees*" name))))
 
-(defun sepia-caller-tree (obj mod)
+(defun sepia-caller-tree (name)
   "Create a tree view of a function's callers.
 
 Pressing RET on a function's name displays its definition.  With
 prefix argument, RET instead visits in another window."
-  (interactive (list (sepia-interactive-arg 'function)
-		     (sepia-interactive-module)))
-  (let* ((defs (xref-defs obj mod))
-	 (mod (if (= (length defs) 1) (or (fourth (car defs)) mod) "*")))
-    (sepia-build-tree-buffer #'xref-callees 
-			     defs
-			     (format "*%s::%s callers*" mod obj))))
-
-(defun sepia-uniquify (xs &optional test)
-  (let ((h (make-hash-table :test (or test #'equal))))
-    (dolist (x xs)
-      (puthash x nil h))
-    (hash-table-keys h)))
-
-(defun sepia-tree-def-order (a b)
-  (or (string< (fourth a) (fourth b))
-      (and (string= (fourth a) (fourth b))
-	   (string< (third a) (third b)))))
+  (interactive (let ((func (sepia-interactive-arg 'function))
+                     (mod (sepia-interactive-module)))
+                 (list (if mod (format "%s::%s" mod func)
+                           func))))
+  (let* ((defs (xref-apropos name)))
+    (sepia-build-tree-buffer
+     #'xref-callers
+     defs (format "*%s callers*" name))))
 
+;;;###autoload
 (defun sepia-module-callee-tree (mod)
   "Display a callee tree for each of MOD's subroutines.
 
 Pressing RET on a function's name displays its definition.  With
 prefix argument, RET instead visits in another window."
   (interactive (list (sepia-interactive-arg 'module)))
-  (let ((defs (sort
-	       (sepia-uniquify
-		(remove-if (lambda (x)
-			     (and (fourth x)
-				  (not (string= (fourth x) mod))))
-			   (mapcan (lambda (x) (xref-defs x mod))
-				   (xref-apropos "" (concat "^" mod "$")))))
-	       #'sepia-tree-def-order)))
+  (let ((defs (xref-mod-subs mod)))
     (sepia-build-tree-buffer #'xref-callees defs (format "*%s subs*" mod))))
 
 (provide 'sepia-tree)
